@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2004 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2005 by Paolo Lucente
 */
 
 /*
@@ -27,18 +27,26 @@
 /* structures */
 struct _dictionary_line {
   char key[SRVBUFLEN];
-  int (*func)(char *, char *);
+  int (*func)(char *, char *, char *);
 };
 
 struct configuration {
+  char *name;
+  char *type;
+  u_int32_t what_to_count;
   int acct_type; 
   int pipe_size;
   int buffer_size;
   int handle_fragments;
+  int handle_flows;
+  int frag_bufsz;
+  int flow_bufsz;
+  int flow_lifetime;
   char *imt_plugin_path;
   char *imt_plugin_passwd;
   char *sql_db;
   char *sql_table;
+  char *sql_table_schema;
   int sql_table_version;
   char *sql_user;
   char *sql_passwd;
@@ -57,6 +65,10 @@ struct configuration {
   int sql_trigger_time;
   int sql_trigger_time_howmany; /* internal */
   char *sql_trigger_exec;
+  int sql_max_queries;
+  char *sql_preprocess;
+  int sql_preprocess_type;
+  int sql_multi_values;
   int print_refresh_time;
   int print_cache_entries;
   int print_markers;
@@ -64,9 +76,12 @@ struct configuration {
   char *nfacctd_ip;
   char *nfacctd_allow_file;
   int nfacctd_time;
+  int nfacctd_as;
   int promisc; /* pcap_open_live() promisc parameter */
   char *clbuf; /* pcap filter */
+  char *pcap_savefile;
   char *dev;
+  int if_wait;
   int num_memory_pools;
   int memory_pool_size;
   int buckets;
@@ -74,15 +89,17 @@ struct configuration {
   int active_plugins;
   char *pidfile; 
   char *networks_file;
-  unsigned int what_to_count;
+  int networks_cache_entries;
+  char *ports_file;
   char *a_filter;
   struct bpf_program bpfp_a_filter;
-  int pre_tag_filter;
+  struct pretag_filter ptf;
   char *pre_tag_map;
   int post_tag;
   int sampling_rate;
   char *syslog;
   int debug;
+  int snaplen;
 };
 
 struct plugin_type_entry {
@@ -106,15 +123,15 @@ struct plugins_list_entry {
 #else
 #define EXT
 #endif
-EXT void evaluate_configuration(int);
+EXT void evaluate_configuration(char *, int);
 EXT int parse_configuration_file(char *);
-EXT int parse_plugin_names(int, int);
-EXT void debug_configuration_file(int);
-EXT int create_plugin(char *, char *);
+EXT int parse_plugin_names(char *, int, int);
+EXT void debug_configuration_file(char *, int);
+EXT int create_plugin(char *, char *, char *);
 EXT int delete_plugin_by_id(int);
 EXT struct plugins_list_entry *search_plugin_by_pipe(int);
 EXT struct plugins_list_entry *search_plugin_by_pid(pid_t);
-EXT void sanitize_cfg(int);
+EXT void sanitize_cfg(int, char *);
 EXT void set_default_values();
 
 /* global vars */
