@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2004 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2005 by Paolo Lucente
 */
 
 /*
@@ -19,6 +19,10 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#if (defined __PKT_HANDLERS_C)
+extern struct channels_list_entry channels_list[MAX_N_PLUGINS]; /* communication channels: core <-> plugins */
+#endif
+
 #if (!defined __PKT_HANDLERS_C)
 #define EXT extern
 #else
@@ -40,18 +44,28 @@ EXT void src_host_handler(struct channels_list_entry *, struct packet_ptrs *, st
 EXT void dst_host_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 EXT void src_port_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 EXT void dst_port_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
+EXT void ip_tos_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 EXT void ip_proto_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 EXT void counters_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 EXT void id_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
+EXT void flows_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 
+EXT void NF_src_mac_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
+EXT void NF_dst_mac_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
+EXT void NF_vlan_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 EXT void NF_src_host_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 EXT void NF_dst_host_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 EXT void NF_src_port_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 EXT void NF_dst_port_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
+EXT void NF_src_as_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
+EXT void NF_dst_as_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
+EXT void NF_ip_tos_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 EXT void NF_ip_proto_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 EXT void NF_counters_msecs_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 EXT void NF_counters_secs_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 EXT void NF_counters_new_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
-EXT void NF_id_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
+EXT void NF_flows_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
+
+EXT void ptag_id_handler(struct channels_list_entry *, struct packet_ptrs *, struct pkt_data *);
 #undef EXT
 
