@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2004 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2005 by Paolo Lucente
 */
 
 /*
@@ -66,26 +66,26 @@ static struct _protocols_struct _protocols[] = {
   {"38", 38}, 
   {"39", 39},
   {"40", 40},
-  {"41", 41},
+  {"ipv6", 41},
   {"42", 42},
-  {"43", 43},
-  {"44", 44},
-  {"46", 45},
+  {"ipv6-route", 43},
+  {"ipv6-frag", 44},
+  {"45", 45},
   {"rsvp", 46},
   {"gre", 47},
   {"48", 48},
   {"49", 49},
-  {"50", 50}, 
-  {"51", 51},
+  {"ipv6-crypt", 50}, 
+  {"ipv6-auth", 51},
   {"52", 52}, 
   {"53", 53},  
   {"54", 54}, 
-  {"55", 55},   
+  {"mobile", 55},   
   {"tlsp", 56},
   {"57", 57}, 
-  {"58", 58},    
-  {"59", 59},
-  {"60", 60},
+  {"ipv6-icmp", 58},    
+  {"ipv6-nonxt", 59},
+  {"ipv6-opts", 60},
   {"61", 61},
   {"62", 62},
   {"63", 63},
@@ -105,7 +105,7 @@ static struct _protocols_struct _protocols[] = {
   {"77", 77},
   {"78", 78}, 
   {"79", 79},  
-  {"80", 80},
+  {"iso-ip", 80},
   {"81", 81}, 
   {"82", 82},
   {"vines", 83},
@@ -127,16 +127,16 @@ static struct _protocols_struct _protocols[] = {
   {"99", 99},
   {"100", 100},  
   {"101", 101}, 
-  {"102", 102},
-  {"103", 103},
+  {"pnni", 102},
+  {"pim", 103},
   {"104", 104},
   {"105", 105},  
   {"106", 106},
   {"107", 107},
-  {"108", 108}, 
+  {"IPcomp", 108}, 
   {"109", 109},
   {"110", 110},  
-  {"111", 111},
+  {"ipx-in-ip", 111},
   {"vrrp", 112},
   {"113", 113}, 
   {"114", 114},
@@ -149,7 +149,7 @@ static struct _protocols_struct _protocols[] = {
   {"121", 121}, 
   {"122", 122},
   {"123", 123},
-  {"124", 123},
+  {"isis", 124},
   {"125", 125},  
   {"126", 126},
   {"127", 127},
@@ -157,7 +157,7 @@ static struct _protocols_struct _protocols[] = {
   {"129", 129},
   {"130", 130},
   {"131", 131}, 
-  {"132", 132},
+  {"sctp", 132},
   {"fc", 133},
   {"", -1},
 };
@@ -165,10 +165,23 @@ static struct _protocols_struct _protocols[] = {
 #ifdef __PMACCTD_C
 static struct _devices_struct _devices[] = {
   {eth_handler, DLT_EN10MB},
-  {fddi_handler, DLT_FDDI},
   {ppp_handler, DLT_PPP},
+  {fddi_handler, DLT_FDDI},
+#if defined DLT_IEEE802_11
   {ieee_802_11_handler, DLT_IEEE802_11}, 
+#endif
+#if defined DLT_LINUX_SLL
   {sll_handler, DLT_LINUX_SLL},
+#endif
+#if defined DLT_RAW
+  {raw_handler, DLT_RAW},
+#endif
+#if defined DLT_C_HDLC
+  {chdlc_handler, DLT_C_HDLC},
+#endif
+#ifdef DLT_HDLC
+  {chdlc_handler, DLT_HDLC},
+#endif
   {NULL, -1},
 };
 #endif
@@ -180,6 +193,7 @@ static struct _dictionary_line dictionary[] = {
   {"pidfile", cfg_key_pidfile},
   {"daemonize", cfg_key_daemonize},
   {"aggregate", cfg_key_aggregate},
+  {"snaplen", cfg_key_snaplen},
   {"aggregate_filter", cfg_key_aggregate_filter},
   {"promisc", cfg_key_promisc},
   {"pcap_filter", cfg_key_pcap_filter},
@@ -187,7 +201,10 @@ static struct _dictionary_line dictionary[] = {
   {"plugin_pipe_size", cfg_key_plugin_pipe_size},
   {"plugin_buffer_size", cfg_key_plugin_buffer_size},
   {"interface", cfg_key_interface},
+  {"interface_wait", cfg_key_interface_wait},
   {"networks_file", cfg_key_networks_file},
+  {"networks_cache_entries", cfg_key_networks_cache_entries},
+  {"ports_file", cfg_key_ports_file},
   {"imt_path", cfg_key_imt_path},
   {"imt_passwd", cfg_key_imt_passwd},
   {"imt_buckets", cfg_key_imt_buckets},
@@ -195,6 +212,7 @@ static struct _dictionary_line dictionary[] = {
   {"imt_mem_pools_size", cfg_key_imt_mem_pools_size},
   {"sql_db", cfg_key_sql_db},
   {"sql_table", cfg_key_sql_table},
+  {"sql_table_schema", cfg_key_sql_table_schema},
   {"sql_table_version", cfg_key_sql_table_version},
   {"sql_host", cfg_key_sql_host},
   {"sql_data", cfg_key_sql_data},
@@ -212,6 +230,9 @@ static struct _dictionary_line dictionary[] = {
   {"sql_trigger_time", cfg_key_sql_trigger_time},
   {"sql_cache_entries", cfg_key_sql_cache_entries},
   {"sql_dont_try_update", cfg_key_sql_dont_try_update},
+  {"sql_preprocess", cfg_key_sql_preprocess},
+  {"sql_preprocess_type", cfg_key_sql_preprocess_type},
+  {"sql_multi_values", cfg_key_sql_multi_values},
   {"print_refresh_time", cfg_key_print_refresh_time},
   {"print_cache_entries", cfg_key_print_cache_entries},
   {"print_markers", cfg_key_print_markers},
@@ -220,8 +241,14 @@ static struct _dictionary_line dictionary[] = {
   {"nfacctd_allow_file", cfg_key_nfacctd_allow_file},
   {"nfacctd_time_secs", cfg_key_nfacctd_time_secs},
   {"nfacctd_time_new", cfg_key_nfacctd_time_new},
+  {"nfacctd_as_new", cfg_key_nfacctd_as_new},
+  {"pmacctd_force_frag_handling", cfg_key_pmacctd_force_frag_handling},
+  {"pmacctd_frag_buffer_size", cfg_key_pmacctd_frag_buffer_size},
+  {"pmacctd_flow_buffer_size", cfg_key_pmacctd_flow_buffer_size},
+  {"pmacctd_flow_lifetime", cfg_key_pmacctd_flow_lifetime},
   {"pmacctd_id", cfg_key_post_tag},	/* obsolete */
   {"nfacctd_id_file", cfg_key_pre_tag_map},	/* obsolete */
+  {"pcap_savefile", cfg_key_pcap_savefile},
   {"pre_tag_map", cfg_key_pre_tag_map},	
   {"pre_tag_filter", cfg_key_pre_tag_filter},
   {"post_tag", cfg_key_post_tag},
